import numpy as np
from numba import jit

@jit(nopython=True)
def DisLTDMin3D(nx,ny,nz,x1,x2,y1,y2,z1,z2,DMin):
    r1 = ((x1-x2   )**2+(y1-y2)**2+(z1-z2   )**2)**0.5
    r2 = ((x1-x2-nx)**2+(y1-y2)**2+(z1-z2   )**2)**0.5
    r3 = ((x1-x2+nx)**2+(y1-y2)**2+(z1-z2   )**2)**0.5
    r4 = ((x1-x2   )**2+(y1-y2)**2+(z1-z2-nz)**2)**0.5
    r5 = ((x1-x2   )**2+(y1-y2)**2+(z1-z2+nz)**2)**0.5
    r6 = ((x1-x2-nx)**2+(y1-y2)**2+(z1-z2-nz)**2)**0.5
    r7 = ((x1-x2-nx)**2+(y1-y2)**2+(z1-z2+nz)**2)**0.5
    r8 = ((x1-x2+nx)**2+(y1-y2)**2+(z1-z2-nz)**2)**0.5
    r9 = ((x1-x2+nx)**2+(y1-y2)**2+(z1-z2+nz)**2)**0.5
    if (r1<=DMin)or(r2<=DMin)or(r3<=DMin)or(r4<=DMin)or(r5<=DMin)\
     or(r6<=DMin)or(r7<=DMin)or(r8<=DMin)or(r9<=DMin):
        lessthanDMin = True
    else: lessthanDMin = False    
    return lessthanDMin
    
@jit(nopython=True)
def DisLTDMin_inPlane(nx,nz,x1,x2,z1,z2,DMin):
    r1 = ((x1-x2   )**2+(z1-z2   )**2)**0.5
    r2 = ((x1-x2-nx)**2+(z1-z2   )**2)**0.5
    r3 = ((x1-x2+nx)**2+(z1-z2   )**2)**0.5
    r4 = ((x1-x2   )**2+(z1-z2-nz)**2)**0.5
    r5 = ((x1-x2   )**2+(z1-z2+nz)**2)**0.5
    r6 = ((x1-x2-nx)**2+(z1-z2-nz)**2)**0.5
    r7 = ((x1-x2-nx)**2+(z1-z2+nz)**2)**0.5
    r8 = ((x1-x2+nx)**2+(z1-z2-nz)**2)**0.5
    r9 = ((x1-x2+nx)**2+(z1-z2+nz)**2)**0.5
    if (r1<=DMin)or(r2<=DMin)or(r3<=DMin)or(r4<=DMin)or(r5<=DMin)\
     or(r6<=DMin)or(r7<=DMin)or(r8<=DMin)or(r9<=DMin):
        lessthanDMin = True
    else: lessthanDMin = False    
    return lessthanDMin

@jit(nopython=True)
def Check_for_Overlap(nx,nz,xSphs,zSphs,RSph,nSph,Gap_P2P_LU):
    Overlap = False
    for i in range(nSph):
        for j in range(nSph):
            if i != j:
                if DisLTDMin_inPlane(nx,nz,xSphs[i],xSphs[j],zSphs[i],zSphs[j],2*RSph+Gap_P2P_LU):
                  Overlap = True 
    return Overlap            

def Set_SphPoss_Random(nx,ny,nz,nSph,RSph,ySphbyR,Gap_P2P_LU):
    xSphs = np.ones(nSph,dtype=np.float64)*np.nan
    ySphs = np.ones(nSph,dtype=np.float64)*np.nan
    zSphs = np.ones(nSph,dtype=np.float64)*np.nan
    ic_max = 10000; ntry = 100
    if nSph>1:
        for itry in range(ntry):
            # print('itry',itry)
            SuccessPlacement = True
            for iS in range(nSph):
                Overlap = True; ic = 0 
                while Overlap and ic<ic_max:
                    xSph = np.random.rand()*(nx-1)
                    zSph = np.random.rand()*(nz-1)
                    Overlap = Check_for_Overlap(nx,nz,xSphs,zSphs,RSph,iS+1,Gap_P2P_LU)
                    ic += 1
                    if ic == ic_max-1:
                        # print('iS,ic',iS,ic)
                        SuccessPlacement = False
                if ic < ic_max-1:    
                    xSphs[iS] = xSph
                    ySphs[iS] = RSph*ySphbyR
                    zSphs[iS] = zSph
                
            if SuccessPlacement : 
                print('success ,itry',itry)
                SphPoss = np.array([xSphs,ySphs,zSphs])
                return SphPoss
        SphPoss = np.ones((3,nSph))*np.nan
        print('Set Shere Pos failed, coverage too high? \n'+\
              'better grid resolution may help because if makes Gap_P2P smaller')
    else: 
        xSphs[0] = (nx-1)/2.
        ySphs[0] = RSph*ySphbyR
        zSphs[0] = (nz-1)/2.
        SphPoss = np.array([xSphs,ySphs,zSphs])
    return SphPoss

def Set_SphPoss(SPs):
    nx = SPs['nx']
    ny = SPs['ny']
    nz = SPs['nz']
    nSph = SPs['nSph']
    RSph = SPs['RSph']
    ySphbyR = SPs['ySphbyR']
    Gap_P2P_LU = SPs['Gap_P2P_LU']
    SphPoss = Set_SphPoss_Random(nx,ny,nz,nSph,RSph,ySphbyR,Gap_P2P_LU)
    return SphPoss


